from core import Request, BankNodeConfig
from bank_protocol.exceptions import InvalidRequest
from services.account_service import deposit_into_account
from utils.constants import MONEY_AMOUNT_MAXIMUM


def account_deposit(request: Request, config: BankNodeConfig):
    if request.body is None:
        raise InvalidRequest("Invalid request format")

    try:
        split_body = request.body.split("/")
        split_ip = split_body[1].split(" ")

        account = split_body[0]
        ip = split_ip[0]
        amount = split_ip[1]
    except IndexError as e:
        raise InvalidRequest("Invalid request format") from e

    if not account.isdigit():
        raise InvalidRequest("Account must be a number")

    account_parsed = int(account)

    if account_parsed < 10_000 or account_parsed > 99_999:
        raise InvalidRequest("Account number out of range")

    if not amount.isdigit():
        raise InvalidRequest("Deposit amount must be a number")

    amount_parsed = int(amount)

    if amount_parsed > MONEY_AMOUNT_MAXIMUM:
        raise InvalidRequest("Cannot deposit this much")

    deposit_into_account(account_parsed, amount_parsed)


__all__ = ["account_deposit"]
