import socket
import logging

from core import Request, Response, BankNodeConfig
from bank_protocol.exceptions import RequestTimeoutError, NoPortsOpenError, HostUnreachableError


class BankProxy():
    def __init__(self, request: Request, address: str, config: BankNodeConfig):
        self.request = request
        self.address = address
        self.config = config

        self.logger = logging.getLogger(__name__)

    def proxy_request(self) -> Response:
        for port in range(self.config.scan_port_start, self.config.scan_port_end + 1):
            self.logger.debug("Connecting to port %d", port)
            try:
                self.config.used_port = port
                self.__proxy_request(port)
                return
            except socket.error as e:
                if e.errno == 111: # Connection refused
                    self.logger.debug("Port %d not open", port)

        self.logger.warning("No ports open on the destination host")
        raise NoPortsOpenError("Destination host has no open ports from range")

    def __proxy_request(self, port):
        pass
