import re

from bank_protocol.exceptions import InvalidRequest


class Request():

    def __init__(self, raw_request: str):
        if re.match(r"^[A-Z]{2}$", raw_request):
            self.command_code = raw_request[0:2] # Still take the first 2 characters, because of lingering crlf
            self.body = None
        elif re.match(r"^[A-Z]{2} .+", raw_request):
            command_code: str = raw_request[0:2]
            body: str = raw_request[3:-1] or ""

            if len(body.split("\n")) > 1:
                raise InvalidRequest("Multiline requests are not supported")

            self.command_code = command_code
            self.body = body
        else:
            raise InvalidRequest("Invalid request")

    def as_request(self):
        """Returns a valid request string with CRLF ending"""
        return str(self) + "\r\n"

    def __str__(self):
        if self.body:
            return f"{self.command_code} {self.body}"
        else:
            return self.command_code


__all__ = ["Request"]
